@@ -27,14 +27,15 @@ CREATE TABLE [radacct] (
 	[AcctAuthentic] [varchar] (32) NULL ,
 	[ConnectInfo_start] [varchar] (32) NULL ,
 	[ConnectInfo_stop] [varchar] (32) NULL ,
-	[AcctInputOctets] [int] NULL ,
-	[AcctOutputOctets] [int] NULL ,
+	[AcctInputOctets] [bigint] NULL ,
+	[AcctOutputOctets] [bigint] NULL ,
 	[CalledStationId] [varchar] (30) NOT NULL ,
 	[CallingStationId] [varchar] (30) NOT NULL ,
 	[AcctTerminateCause] [varchar] (32) NOT NULL ,
 	[ServiceType] [varchar] (32) NULL ,
 	[FramedProtocol] [varchar] (32) NULL ,
 	[FramedIPAddress] [varchar] (15) NOT NULL ,
+	[XAscendSessionSvrKey] [varchar] (10) NOT NULL ,
 	[AcctStartDelay] [int] NULL ,
 	[AcctStopDelay] [int] NULL
 ) ON [PRIMARY]
