@@ -4,7 +4,7 @@
  * radiusd.h	Structures, prototypes and global variables
  *		for the FreeRADIUS server.
  *
- * Version:	$Id: radiusd.h,v 1.156.2.4 2005/04/11 23:45:21 aland Exp $
+ * Version:	$Id: radiusd.h,v 1.156.2.4.2.2 2007/06/12 09:31:45 aland Exp $
  *
  */
 #include "libradius.h"
@@ -85,6 +85,8 @@ typedef struct auth_req {
 
 	int			finished;
 	int			options; /* miscellanous options */
+	const char		*component;
+	const char		*module;
 	void			*container;
 } REQUEST;
 
@@ -405,13 +407,13 @@ void		xlat_unregister(const char *module, RAD_XLAT_FUNC func);
 extern		int thread_pool_init(void);
 extern		int thread_pool_clean(time_t now);
 extern		int thread_pool_addrequest(REQUEST *, RAD_REQUEST_FUNP);
-extern		pid_t rad_fork(int exec_wait);
-extern		pid_t rad_waitpid(pid_t pid, int *status, int options);
+extern		pid_t rad_fork(void);
+extern		pid_t rad_waitpid(pid_t pid, int *status);
 extern          int total_active_threads(void);
 
 #ifndef HAVE_PTHREAD_H
 #define rad_fork(n) fork()
-#define rad_waitpid waitpid
+#define rad_waitpid(a,b) waitpid(a,b,0)
 #endif
 
 /* mainconfig.h */
