@@ -11,7 +11,7 @@
  *  This implementation can do ~10^6 lookups/s, which should be fast
  *  enough for most people.
  *
- * Version:	$Id: hash.c,v 1.9.2.9 2006/11/16 00:44:56 aland Exp $
+ * Version:	$Id: hash.c,v 1.9.2.10 2007/02/09 15:06:01 aland Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -30,7 +30,7 @@
  *  Copyright 2005  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: hash.c,v 1.9.2.9 2006/11/16 00:44:56 aland Exp $";
+static const char rcsid[] = "$Id: hash.c,v 1.9.2.10 2007/02/09 15:06:01 aland Exp $";
 
 #include "autoconf.h"
 
@@ -551,20 +551,25 @@ int lrad_hash_table_delete(lrad_hash_table_t *ht, const void *data)
  */
 void lrad_hash_table_free(lrad_hash_table_t *ht)
 {
+	int i;
 	lrad_hash_entry_t *node, *next;
 
 	if (!ht) return;
 
 	/*
-	 *	The entries MUST be all in one linked list.
+	 *	Walk over the buckets, freeing them all.
 	 */
-	for (node = ht->buckets[0]; node != &ht->null; node = next) {
-		next = node->next;
-
-		if (!node->data) continue; /* dummy entry */
-
-		if (ht->free) ht->free(node->data);
-		free(node);
+	for (i = 0; i < ht->num_buckets; i++) {
+		if (ht->buckets[i]) for (node = ht->buckets[i];
+					 node != &ht->null;
+					 node = next) {
+			next = node->next;
+			
+			if (!node->data) continue; /* dummy entry */
+			
+			if (ht->free) ht->free(node->data);
+			free(node);
+		}
 	}
 
 	free(ht->buckets);
@@ -792,7 +797,7 @@ uint32_t lrad_hash_string(const char *p)
 
 #ifdef TESTING
 /*
- *  cc -DTESTING -I .. hash.c -o hash
+ *  cc -g -DTESTING -I ../include hash.c -o hash
  *
  *  ./hash
  */
