@@ -2,7 +2,7 @@
  * token.c	Read the next token from a string.
  *		Yes it's pretty primitive but effective.
  *
- * Version:	$Id: token.c,v 1.17.2.1 2005/08/26 12:54:00 nbk Exp $
+ * Version:	$Id: token.c,v 1.17.2.1.2.1 2006/03/15 15:37:58 nbk Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,7 @@
  *
  *   You should have received a copy of the GNU Lesser General Public
  *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  *
  * Copyright 2000  The FreeRADIUS server project
  */
@@ -27,7 +27,7 @@
 #include <ctype.h>
 #include "token.h"
 
-static const char rcsid[] = "$Id: token.c,v 1.17.2.1 2005/08/26 12:54:00 nbk Exp $";
+static const char rcsid[] = "$Id: token.c,v 1.17.2.1.2.1 2006/03/15 15:37:58 nbk Exp $";
 
 static const LRAD_NAME_NUMBER tokens[] = {
 	{ "=~", T_OP_REG_EQ,	}, /* order is important! */
