@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id: auth.c,v 1.136.2.5 2005/10/31 17:59:21 nbk Exp $
+ * Version:	$Id: auth.c,v 1.136.2.5.2.2 2007/03/05 14:30:04 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  */
 
-static const char rcsid[] = "$Id: auth.c,v 1.136.2.5 2005/10/31 17:59:21 nbk Exp $";
+static const char rcsid[] = "$Id: auth.c,v 1.136.2.5.2.2 2007/03/05 14:30:04 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -245,6 +245,19 @@ int rad_check_password(REQUEST *request)
 		if (auth_type == -1) auth_type = PW_AUTHTYPE_CRYPT;
 	} else {
 		password_pair = pairfind(request->config_items, PW_PASSWORD);
+		if (!password_pair) {
+			password_pair = pairfind(request->config_items,
+						 PW_CLEARTEXT_PASSWORD);
+			if (password_pair) {
+				VALUE_PAIR *old;
+
+				old = pairmake("User-Password",
+					       password_pair->strvalue,
+					       T_OP_SET);
+				if (old) pairadd(&request->config_items, old);
+			}
+		}
+		
 	}
 
 	if (auth_type < 0) {
@@ -876,6 +889,7 @@ autz_redo:
 		pairdelete(&request->reply->vps, PW_EXEC_PROGRAM);
 	}
 	if ((auth_item = pairfind(request->reply->vps, PW_EXEC_PROGRAM_WAIT)) != NULL) {
+		free(exec_program);
 		exec_wait = 1;
 		exec_program = strdup((char *)auth_item->strvalue);
 		pairdelete(&request->reply->vps, PW_EXEC_PROGRAM_WAIT);
