@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.100.2.1.2.1 2006/04/19 16:21:50 aland Exp $
+ * Version:	$Id: conffile.c,v 1.100.2.1.2.6 2007/04/07 22:28:17 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -36,6 +36,10 @@
 #	include <netinet/in.h>
 #endif
 
+#ifdef HAVE_SYS_STAT_H
+#include	<sys/stat.h>
+#endif
+
 #include "radiusd.h"
 #include "rad_assert.h"
 #include "conffile.h"
@@ -43,7 +47,7 @@
 #include "modules.h"
 
 static const char rcsid[] =
-"$Id: conffile.c,v 1.100.2.1.2.1 2006/04/19 16:21:50 aland Exp $";
+"$Id: conffile.c,v 1.100.2.1.2.6 2007/04/07 22:28:17 aland Exp $";
 
 #define xstrdup strdup
 
@@ -68,6 +72,8 @@ struct conf_part {
 	CONF_ITEM item;
 	char *name1;
 	char *name2;
+	void *base;
+	const CONF_PARSER *variables;
 	struct conf_item *children;
 };
 
@@ -140,11 +146,54 @@ void cf_pair_free(CONF_PAIR **cp)
 	*cp = NULL;
 }
 
+/*
+ *	Free strings we've parsed into data structures.
+ */
+static void cf_section_parse_free(void *base, const CONF_PARSER *variables)
+{
+	int i;
+
+	/*
+	 *	Don't automatically free the strings if we're being
+	 *	called from a module.
+	 */
+	if (base || !variables) return;
+	
+	/*
+	 *	Free up dynamically allocated string pointers.
+	 */
+	for (i = 0; variables[i].name != NULL; i++) {
+		char **p;
+
+		if (variables[i].type != PW_TYPE_STRING_PTR) {
+			continue;
+		}
+
+		/*
+		 *	Prefer the data, if it's there.
+		 *	Else use the base + offset.
+		 */
+		if (!variables[i].data) {
+			continue;
+		}
+
+		/*
+		 *	FIXME: Add base
+		 */
+		p = (char **) (variables[i].data);
+
+
+		free(*p);
+		*p = NULL;
+	}
+}
+
+
 /*
  *	Allocate a CONF_SECTION
  */
 static CONF_SECTION *cf_section_alloc(const char *name1, const char *name2,
-		CONF_SECTION *parent)
+				      CONF_SECTION *parent, const char *cf)
 {
 	CONF_SECTION	*cs;
 
@@ -170,6 +219,10 @@ void cf_section_free(CONF_SECTION **cs)
 
 	if (!cs || !*cs) return;
 
+	if ((*cs)->variables) {
+		cf_section_parse_free((*cs)->base, (*cs)->variables);
+	}
+
 	for (ci = (*cs)->children; ci; ci = next) {
 		next = ci->next;
 		if (ci->type==CONF_ITEM_PAIR) {
@@ -413,6 +466,7 @@ static const char *cf_expand_variables(const char *cf, int *lineno,
 	return output;
 }
 
+
 /*
  *	Parse a configuration section into user-supplied variables.
  */
@@ -464,8 +518,9 @@ int cf_section_parse(CONF_SECTION *cs, void *base,
 			}
 
 			rcode = cf_section_parse(subsection, base,
-					(CONF_PARSER *) data);
+						 (CONF_PARSER *) data);
 			if (rcode < 0) {
+				cf_section_parse_free(base, variables);
 				return -1;
 			}
 			break;
@@ -483,6 +538,7 @@ int cf_section_parse(CONF_SECTION *cs, void *base,
 			} else {
 				*(int *)data = 0;
 				radlog(L_ERR, "Bad value \"%s\" for boolean variable %s", value, variables[i].name);
+				cf_section_parse_free(base, variables);
 				return -1;
 			}
 			DEBUG2(" %s: %s = %s",
@@ -515,6 +571,7 @@ int cf_section_parse(CONF_SECTION *cs, void *base,
 							    &cs->item.lineno,
 							    cs, buffer, value);
 				if (!value) {
+					cf_section_parse_free(base, variables);
 					return -1;
 				}
 			}
@@ -537,6 +594,7 @@ int cf_section_parse(CONF_SECTION *cs, void *base,
 			ipaddr = ip_getaddr(value);
 			if (ipaddr == 0) {
 				radlog(L_ERR, "Can't find IP address for host %s", value);
+				cf_section_parse_free(base, variables);
 				return -1;
 			}
 			DEBUG2(" %s: %s = %s IP address [%s]",
@@ -548,14 +606,19 @@ int cf_section_parse(CONF_SECTION *cs, void *base,
 
 		default:
 			radlog(L_ERR, "type %d not supported yet", variables[i].type);
+			cf_section_parse_free(base, variables);
 			return -1;
 			break;
 		} /* switch over variable type */
 	} /* for all variables in the configuration section */
 
+	cs->base = base;
+	cs->variables = variables;
+
 	return 0;
 }
 
+
 /*
  *	Read a part of the config file.
  */
@@ -588,7 +651,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 	/*
 	 *	Allocate new section.
 	 */
-	cs = cf_section_alloc(name1, name2, parent);
+	cs = cf_section_alloc(name1, name2, parent, cf);
 	cs->item.lineno = *lineno;
 
 	/*
@@ -609,8 +672,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 		 *	We've filled the buffer, and there isn't
 		 *	a CR in it.  Die!
 		 */
-		if ((len == sizeof(buf)) &&
-		    (cbuf[len - 1] != '\n')) {
+		if (cbuf[len - 1] != '\n') {
 			radlog(L_ERR, "%s[%d]: Line too long",
 			       cf, *lineno);
 			cf_section_free(&cs);
@@ -627,6 +689,13 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 		 *	and read another line.
 		 */
 		if ((len > 0) && (cbuf[len - 1] == '\\')) {
+			if (len >= (sizeof(buf) - 5)) {
+				radlog(L_ERR, "%s[%d]: Line too long",
+				       cf, *lineno);
+				cf_section_free(&cs);
+				return NULL;
+			}
+
 			cbuf[len - 1] = '\0';
 			cbuf += len - 1;
 			continue;
@@ -817,23 +886,35 @@ CONF_SECTION *conf_read(const char *fromfile, int fromline,
 	FILE		*fp;
 	int		lineno = 0;
 	CONF_SECTION	*cs;
+	struct stat	statbuf;
+	char		buf[8192];
+
+	buf[0] = '\0';
+	if (fromfile) {
+		snprintf(buf, sizeof(buf), "%s[%d]: ", fromfile, fromline);
+	}
+
+	if (stat(conffile, &statbuf) == 0) {
+		if ((statbuf.st_mode & S_IWOTH) != 0) {
+			radlog(L_ERR|L_CONS, "%sConfiguration file %s is globally writable.  Refusing to start due to insecure configuration.",
+			       buf[0] ? buf : "", conffile);
+			return NULL;
+		}
+
+		if (0 && (statbuf.st_mode & S_IROTH) != 0) {
+			radlog(L_ERR|L_CONS, "%sConfiguration file %s is globally readable.  Refusing to start due to insecure configuration.",
+			       buf[0] ? buf : "", conffile);
+			return NULL;
+		}
+	}
 
 	if ((fp = fopen(conffile, "r")) == NULL) {
-		if (fromfile) {
-			radlog(L_ERR|L_CONS, "%s[%d]: Unable to open file \"%s\": %s",
-					fromfile, fromline, conffile, strerror(errno));
-		} else {
-			radlog(L_ERR|L_CONS, "Unable to open file \"%s\": %s",
-					conffile, strerror(errno));
-		}
+		radlog(L_ERR|L_CONS, "%sUnable to open file \"%s\": %s",
+		       buf[0] ? buf : "", conffile, strerror(errno));
 		return NULL;
 	}
 
-	if(parent) {
-	    cs = cf_section_read(conffile, &lineno, fp, NULL, NULL, parent);
-	} else {
-	    cs = cf_section_read(conffile, &lineno, fp, NULL, NULL, NULL);
-	}
+	cs = cf_section_read(conffile, &lineno, fp, NULL, NULL, parent);
 
 	fclose(fp);
 
