@@ -1,7 +1,7 @@
 /*
  * modcall.c
  *
- * Version:	$Id: modcall.c,v 1.22.2.1.2.3 2005/12/15 00:44:34 aland Exp $
+ * Version:	$Id: modcall.c,v 1.22.2.1.2.4 2007/06/12 09:31:45 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -232,10 +232,13 @@ static int call_modsingle(int component, modsingle *sp, REQUEST *request,
 	DEBUG3("  modsingle[%s]: calling %s (%s) for request %d",
 	       comp2str[component], sp->modinst->name,
 	       sp->modinst->entry->name, request->number);
+	request->component = comp2str[component];
+	request->module = sp->modinst->entry->name;
 	safe_lock(sp->modinst);
 	myresult = sp->modinst->entry->module->methods[component](
 			sp->modinst->insthandle, request);
 	safe_unlock(sp->modinst);
+	request->module = "<server core>";
 	DEBUG3("  modsingle[%s]: returned from %s (%s) for request %d",
 	       comp2str[component], sp->modinst->name,
 	       sp->modinst->entry->name, request->number);
