@@ -1,7 +1,7 @@
 /*
  * radclient.c	General radius packet debug tool.
  *
- * Version:	$Id: radclient.c,v 1.72.2.1.2.5 2006/05/16 18:26:08 aland Exp $
+ * Version:	$Id: radclient.c,v 1.72.2.1.2.7 2007/04/07 22:22:51 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2000  Miquel van Smoorenburg <miquels@cistron.nl>
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
-static const char rcsid[] = "$Id: radclient.c,v 1.72.2.1.2.5 2006/05/16 18:26:08 aland Exp $";
+static const char rcsid[] = "$Id: radclient.c,v 1.72.2.1.2.7 2007/04/07 22:22:51 aland Exp $";
 
 #include "autoconf.h"
 
@@ -191,6 +191,7 @@ static radclient_t *radclient_init(const char *filename)
 		radclient = malloc(sizeof(*radclient));
 		if (!radclient) {
 			perror("radclient: X");
+			if (fp != stdin) fclose(fp);
 			return NULL; /* memory leak "start" */
 		}
 		memset(radclient, 0, sizeof(*radclient));
@@ -199,6 +200,7 @@ static radclient_t *radclient_init(const char *filename)
 		if (!radclient->request) {
 			librad_perror("radclient: X");
 			radclient_free(radclient);
+			if (fp != stdin) fclose(fp);
 			return NULL; /* memory leak "start" */
 		}
 
@@ -212,6 +214,7 @@ static radclient_t *radclient_init(const char *filename)
 		radclient->request->vps = readvp2(fp, &filedone, "radclient: X");
 		if (!radclient->request->vps) {
 			radclient_free(radclient);
+			if (fp != stdin) fclose(fp);
 			return start; /* done: return the list */
 		}
 
@@ -612,7 +615,8 @@ static int recv_one_packet(int wait_time)
 	 */
 	reply = rad_recv(sockfd);
 	if (!reply) {
-		fprintf(stderr, "radclient: received bad packet\n");
+		fprintf(stderr, "radclient: received bad packet: %s\n",
+			librad_errstr);
 		return -1;	/* bad packet */
 	}
 
@@ -794,7 +798,7 @@ int main(int argc, char **argv)
 			timeout = atof(optarg);
 			break;
 		case 'v':
-			printf("radclient: $Id: radclient.c,v 1.72.2.1.2.5 2006/05/16 18:26:08 aland Exp $ built on " __DATE__ " at " __TIME__ "\n");
+			printf("radclient: $Id: radclient.c,v 1.72.2.1.2.7 2007/04/07 22:22:51 aland Exp $ built on " __DATE__ " at " __TIME__ "\n");
 			exit(0);
 			break;
 		case 'x':
