@@ -1,7 +1,7 @@
 /*
  * radiusd.c	Main loop of the radius server.
  *
- * Version:	$Id: radiusd.c,v 1.321.2.2.2.3 2006/05/16 18:26:07 aland Exp $
+ * Version:	$Id: radiusd.c,v 1.321.2.2.2.5 2007/02/12 09:55:24 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 /* don't look here for the version, run radiusd -v or look in version.c */
 static const char rcsid[] =
-"$Id: radiusd.c,v 1.321.2.2.2.3 2006/05/16 18:26:07 aland Exp $";
+"$Id: radiusd.c,v 1.321.2.2.2.5 2007/02/12 09:55:24 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -105,6 +105,7 @@ static int dont_fork = FALSE;
 static time_t start_time = 0;
 static int spawn_flag = TRUE;
 static int do_exit = 0;
+static int debug_memory = 0;
 
 /*
  *	Static functions.
@@ -826,7 +827,7 @@ int main(int argc, char *argv[])
 #endif
 
 	/*  Process the options.  */
-	while ((argval = getopt(argc, argv, "Aa:bcd:fg:hi:l:p:sSvxXyz")) != EOF) {
+	while ((argval = getopt(argc, argv, "Aa:bcd:fg:hi:l:mp:sSvxXyz")) != EOF) {
 
 		switch(argval) {
 
@@ -876,6 +877,10 @@ int main(int argc, char *argv[])
 				syslog_facility = str2fac(optarg);
 				break;
 
+			case 'm':
+				debug_memory = 1;
+				break;
+
 			case 'S':
 				log_stripped_names++;
 				break;
@@ -1137,7 +1142,7 @@ int main(int argc, char *argv[])
 	 *	server to die immediately.  Use SIGTERM to shut down
 	 *	the server cleanly in that case.
 	 */
-	if (debug_flag == 0) {
+	if ((debug_memory == 1) || (debug_flag == 0)) {
 #ifdef HAVE_SIGACTION
 	        act.sa_handler = sig_fatal;
 		sigaction(SIGINT, &act, NULL);
@@ -1203,6 +1208,7 @@ int main(int argc, char *argv[])
 			 *	Free the configuration items.
 			 */
 			free_mainconfig();
+			free(radius_dir);
 
 			/*
 			 *	SIGTERM gets do_exit=0,
@@ -1424,7 +1430,17 @@ int main(int argc, char *argv[])
 				}
 			} else
 #endif
-				rad_respond(request, fun);
+			  {
+				  rad_respond(request, fun);
+
+				  /*
+				   *	Requests that care about child
+				   *	process exit codes have already
+				   *	either called rad_waitpid(), or
+				   *	they've given up.
+				   */
+				  wait(NULL);
+			  }
 		} /* loop over listening sockets*/
 
 #ifdef WITH_SNMP
@@ -1938,6 +1954,13 @@ static void sig_fatal(int sig)
 		case SIGTERM:
 			do_exit = 1;
 			break;
+		case SIGINT:
+		case SIGQUIT:
+			if (debug_memory) {
+				do_exit = 1;
+				break;
+			}
+
 		default:
 			do_exit = 2;
 			break;
