@@ -2,7 +2,7 @@
  * request_list.c	Hide the handling of the REQUEST list from
  *			the main server.
  *
- * Version:	$Id: request_list.c,v 1.40.2.2.2.6 2007/03/05 14:15:32 aland Exp $
+ * Version:	$Id: request_list.c,v 1.40.2.2.2.7 2007/06/12 09:31:46 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *
  * Copyright 2003-2004  The FreeRADIUS server project
  */
-static const char rcsid[] = "$Id: request_list.c,v 1.40.2.2.2.6 2007/03/05 14:15:32 aland Exp $";
+static const char rcsid[] = "$Id: request_list.c,v 1.40.2.2.2.7 2007/06/12 09:31:46 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -1128,8 +1128,9 @@ static int refresh_request(REQUEST *request, void *data)
 			 */
 		} else if ((child_pid != NO_SUCH_CHILD_PID) &&
 			   ((request->options & RAD_REQUEST_OPTION_LOGGED_CHILD) == 0)) {
-			radlog(L_ERR, "WARNING: Unresponsive child (id %lu) for request %d",
-			       (unsigned long)child_pid, number);
+			radlog(L_ERR, "WARNING: Unresponsive child (id %lu) for request %d (in component %s module %s)",
+			       (unsigned long)child_pid, number,
+			       request->component, request->module);
 
 			/*
 			 *  Set the option that we've sent a log message,
