@@ -1,7 +1,7 @@
 /*
  * session.c	session management
  *
- * Version:	$Id: session.c,v 1.21.2.1 2005/03/17 13:52:44 phampson Exp $
+ * Version:	$Id: session.c,v 1.21.2.1.2.3 2007/05/15 09:50:51 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -59,6 +59,7 @@ int session_zap(REQUEST *request, uint32_t nasaddr, unsigned int port,
 	/* Hold your breath */
 #define PAIR(n,v,t,e) do { \
 		if(!(vp = paircreate(n, t))) { \
+			request_free(&stopreq); \
 			radlog(L_ERR|L_CONS, "no memory"); \
 			pairfree(&(stopreq->packet->vps)); \
 			return 0; \
@@ -70,6 +71,7 @@ int session_zap(REQUEST *request, uint32_t nasaddr, unsigned int port,
 #define IPPAIR(n,v) PAIR(n,v,PW_TYPE_IPADDR,lvalue)
 #define STRINGPAIR(n,v) do { \
 	if(!(vp = paircreate(n, PW_TYPE_STRING))) { \
+		request_free(&stopreq); \
 		radlog(L_ERR|L_CONS, "no memory"); \
 		pairfree(&(stopreq->packet->vps)); \
 		return 0; \
@@ -155,27 +157,13 @@ int rad_check_ts(uint32_t nasaddr, unsigned int portnum, const char *user,
 	/*
 	 *	Fork.
 	 */
-	if ((pid = rad_fork(1)) < 0) { /* do wait for the fork'd result */
+	if ((pid = rad_fork()) < 0) { /* do wait for the fork'd result */
 		radlog(L_ERR, "Accounting: Failed in fork(): Cannot run checkrad\n");
 		return -1;
 	}
 
 	if (pid > 0) {
-		int found = 0;
-
-		/*
-		 *	Parent - Wait for checkrad to terminate.
-		 *	We timeout in 10 seconds.
-		 */
-		child_pid = -1;
-		for (n = 0; n < 10; n++) {
-			sleep(1);
-			child_pid = rad_waitpid(pid, &status, WNOHANG);
-			if ((child_pid < 0) || (child_pid == pid)) {
-				found = 1;
-				break;
-			}
-		}
+		child_pid = rad_waitpid(pid, &status);
 
 		/*
 		 *	It's taking too long.  Stop waiting for it.
@@ -183,7 +171,7 @@ int rad_check_ts(uint32_t nasaddr, unsigned int portnum, const char *user,
 		 *	Don't bother to kill it, as we don't care what
 		 *	happens to it now.
 		 */
-		if (!found) {
+		if (child_pid == 0) {
 			radlog(L_ERR, "Check-TS: timeout waiting for checkrad");
 			return 2;
 		}
@@ -196,11 +184,7 @@ int rad_check_ts(uint32_t nasaddr, unsigned int portnum, const char *user,
 		return WEXITSTATUS(status);
 	}
 
-	/*
-	 *	Child - exec checklogin with the right parameters.
-	 */
-	for (n = 256; n >= 3; n--)
-		close(n);
+	closefrom(3);
 
 	/*
 	 *  We don't close fd's 0, 1, and 2.  If we're in debugging mode,
