@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: threads.c,v 1.77.2.7.2.2 2007/02/14 15:44:22 aland Exp $
+ * Version:	$Id: threads.c,v 1.77.2.7.2.3 2007/05/16 07:40:41 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@
 #include "conffile.h"
 
 static const char rcsid[] =
-"$Id: threads.c,v 1.77.2.7.2.2 2007/02/14 15:44:22 aland Exp $";
+"$Id: threads.c,v 1.77.2.7.2.3 2007/05/16 07:40:41 aland Exp $";
 
 #ifdef HAVE_OPENSSL_CRYPTO_H
 #include <openssl/crypto.h>
@@ -818,8 +818,13 @@ int thread_pool_init(void)
 	}
 
 	pool_cf = cf_section_find("thread");
-	if (pool_cf != NULL) {
-		cf_section_parse(pool_cf, NULL, thread_config);
+	if (!pool_cf) {
+		radlog(L_ERR, "FATAL: Attempting to start in multi-threaded mode with no thread configuration in radiusd.conf");
+		exit(1);
+	}
+
+	if (cf_section_parse(pool_cf, NULL, thread_config) < 0) {
+		exit(1);
 	}
 
 	/*
