@@ -1,7 +1,7 @@
 /*
  * util.c	Various utility functions.
  *
- * Version:     $Id: util.c,v 1.37.2.1 2005/07/14 17:53:36 nbk Exp $
+ * Version:     $Id: util.c,v 1.37.2.1.2.2 2007/06/12 09:31:46 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: util.c,v 1.37.2.1 2005/07/14 17:53:36 nbk Exp $";
+static const char rcsid[] = "$Id: util.c,v 1.37.2.1.2.2 2007/06/12 09:31:46 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -325,7 +325,7 @@ void xfree(const char *ptr)
  *
  */
 
-void rad_assert_fail (const char *file, unsigned int line)
+void NEVER_RETURNS rad_assert_fail (const char *file, unsigned int line)
 {
 	radlog(L_ERR|L_CONS, "Assertion failed in %s, line %u", file, line);
 	abort();
@@ -354,6 +354,8 @@ REQUEST *request_alloc(void)
 	request->child_pid = NO_SUCH_CHILD_PID;
 	request->container = NULL;
 	request->options = RAD_REQUEST_OPTION_NONE;
+	request->component = "<server core>";
+	request->module = "<server core>";
 
 	return request;
 }
