@@ -2,7 +2,7 @@
  * valuepair.c	Valuepair functions that are radiusd-specific
  *		and as such do not belong in the library.
  *
- * Version:	$Id: valuepair.c,v 1.60.2.4 2005/02/07 20:04:47 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.60.2.4.2.1 2007/02/09 15:56:12 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: valuepair.c,v 1.60.2.4 2005/02/07 20:04:47 aland Exp $";
+static const char rcsid[] = "$Id: valuepair.c,v 1.60.2.4.2.1 2007/02/09 15:56:12 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -736,6 +736,18 @@ void pair_builtincompare_init(void)
 	paircompare_register(PW_RESPONSE_PACKET_TYPE, 0, responsecmp, NULL);
 }
 
+void paircompare_builtin_free(void)
+{
+	struct cmp *c, *next;
+	
+	for (c = cmp; c != NULL; c = next) {
+		next = c->next;
+		free(c);
+	}
+}
+
+
+
 /*
  *	Move pairs, replacing/over-writing them, and doing xlat.
  */
