@@ -2,7 +2,7 @@
  * xlat.c	Translate strings.  This is the first version of xlat
  * 		incorporated to RADIUS
  *
- * Version:	$Id: xlat.c,v 1.72.2.7.2.4 2007/02/09 15:56:13 aland Exp $
+ * Version:	$Id: xlat.c,v 1.72.2.7.2.5 2007/07/14 11:07:54 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 static const char rcsid[] =
-"$Id: xlat.c,v 1.72.2.7.2.4 2007/02/09 15:56:13 aland Exp $";
+"$Id: xlat.c,v 1.72.2.7.2.5 2007/07/14 11:07:54 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -340,6 +340,11 @@ int xlat_register(const char *module, RAD_XLAT_FUNC func, void *instance)
 			c->internal = TRUE;
 		}
 
+		xlat_register("control", xlat_packet, &xlat_inst[0]);
+		c = xlat_find("control");
+		rad_assert(c != NULL);
+		c->internal = TRUE;
+
 #ifdef HAVE_REGEX_H
 		/*
 		 *	Register xlat's for regexes.
