@@ -3,7 +3,7 @@
  *                      before sending reply to the NAS/Server that sent
  *                      it to us.
  *
- * Version:      $Id: rlm_attr_filter.c,v 1.19.2.1 2005/08/11 21:06:38 aland Exp $
+ * Version:      $Id: rlm_attr_filter.c,v 1.19.2.1.2.1 2007/05/16 07:44:18 aland Exp $
  *
  *   This program is is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License, version 2 if the
@@ -41,7 +41,7 @@
 #include	"radiusd.h"
 #include	"modules.h"
 
-static const char rcsid[] = "$Id: rlm_attr_filter.c,v 1.19.2.1 2005/08/11 21:06:38 aland Exp $";
+static const char rcsid[] = "$Id: rlm_attr_filter.c,v 1.19.2.1.2.1 2007/05/16 07:44:18 aland Exp $";
 
 struct attr_filter_instance {
         /* autz */
@@ -272,7 +272,6 @@ static int attr_filter_authorize(void *instance, REQUEST *request)
 	int             compare;
 	int             pass, fail;
 	VALUE_PAIR      *realmpair;
-	REALM           *realm;
 	char            *realmname;
 
 	/*
@@ -301,7 +300,6 @@ static int attr_filter_authorize(void *instance, REQUEST *request)
 	}
 
 	realmname = (char *) realmpair->strvalue;
-	realm = realm_find(realmname, FALSE);
 
 	/*
 	 *      Find the attr_filter profile entry for the realm.
@@ -417,7 +415,6 @@ static int attr_filter_accounting(void *instance, REQUEST *request)
 	int		compare;
 	int		pass, fail;
 	VALUE_PAIR	*realmpair;
-	REALM		*realm;
 	char		*realmname;
 	/*
 	 * Accounting is similar to pre-proxy.
@@ -441,7 +438,6 @@ static int attr_filter_accounting(void *instance, REQUEST *request)
 	}
 
 	realmname = (char *) realmpair->strvalue;
-	realm = realm_find (realmname, FALSE);
 
 	/*
 	 * Find the attr_filter profile entry for the realm
@@ -544,7 +540,6 @@ static int attr_filter_preproxy (void *instance, REQUEST *request)
 	int		compare;
 	int		pass, fail;
 	VALUE_PAIR	*realmpair;
-	REALM		*realm;
 	char		*realmname;
 
 	/*
@@ -564,7 +559,6 @@ static int attr_filter_preproxy (void *instance, REQUEST *request)
 	}
 
 	realmname = (char *)realmpair->strvalue;
-	realm = realm_find(realmname, FALSE);
 
 	for (pl = inst->attrs; pl; pl = pl->next) {
 		if ( (strcmp(pl->name, "DEFAULT") != 0) &&
@@ -655,7 +649,6 @@ static int attr_filter_postproxy(void *instance, REQUEST *request)
 	int		compare;
 	int		pass, fail = 0;
 	VALUE_PAIR	*realmpair;
-	REALM		*realm;
 	char		*realmname;
 	/*
 	 *	It's not a proxy reply, so return NOOP
@@ -684,8 +677,6 @@ static int attr_filter_postproxy(void *instance, REQUEST *request)
 
 	realmname = (char *) realmpair->strvalue;
 
-	realm = realm_find(realmname, FALSE);
-
 	/*
 	 *      Find the attr_filter profile entry for the realm.
 	 */
