@@ -1,7 +1,7 @@
 /*
  * rlm_detail.c	accounting:    Write the "detail" files.
  *
- * Version:	$Id: rlm_detail.c,v 1.37.2.1.2.2 2006/07/05 17:08:31 aland Exp $
+ * Version:	$Id: rlm_detail.c,v 1.37.2.1.2.3 2007/03/05 14:43:31 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: rlm_detail.c,v 1.37.2.1.2.2 2006/07/05 17:08:31 aland Exp $";
+static const char rcsid[] = "$Id: rlm_detail.c,v 1.37.2.1.2.3 2007/03/05 14:43:31 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -198,7 +198,7 @@ static int do_detail(void *instance, REQUEST *request, RADIUS_PACKET *packet,
 	struct timeval	tv;
 	REALM		*proxy_realm;
 	char		proxy_buffer[16];
-	VALUE_PAIR	*pair = packet->vps;
+	VALUE_PAIR	*pair;
 
 	struct detail_instance *inst = instance;
 
@@ -353,7 +353,7 @@ static int do_detail(void *instance, REQUEST *request, RADIUS_PACKET *packet,
 	fputs(CTIME_R(&request->timestamp, buffer, DIRLEN), outfp);
 
 	/* Write each attribute/value to the log file */
-	for (; pair != NULL; pair = pair->next) {
+	for (pair = packet->vps; pair != NULL; pair = pair->next) {
 		if (inst->ht &&
 		    lrad_hash_table_finddata(inst->ht, pair)) continue;
 
