@@ -1,7 +1,7 @@
 /*
  * rlm_chap.c
  *
- * Version:  $Id: rlm_digest.c,v 1.5.4.6 2006/11/27 20:44:19 aland Exp $
+ * Version:  $Id: rlm_digest.c,v 1.5.4.7 2007/07/17 18:29:20 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 #include "modules.h"
 #include "conffile.h"
 
-static const char rcsid[] = "$Id: rlm_digest.c,v 1.5.4.6 2006/11/27 20:44:19 aland Exp $";
+static const char rcsid[] = "$Id: rlm_digest.c,v 1.5.4.7 2007/07/17 18:29:20 aland Exp $";
 
 static int digest_authorize(void *instance, REQUEST *request)
 {
@@ -103,7 +103,9 @@ static int digest_authenticate(void *instance, REQUEST *request)
 			return RLM_MODULE_INVALID;
 		}
 	} else {
-		passwd = pairfind(request->config_items, PW_PASSWORD);
+		passwd = pairfind(request->config_items, PW_CLEARTEXT_PASSWORD);
+		if (!passwd) passwd = pairfind(request->config_items,
+					       PW_USER_PASSWORD);
 	}
 	if (!passwd) {
 		radlog(L_AUTH, "rlm_digest: Configuration item \"User-Password\" or Digest-HA1 is required for authentication.");
