@@ -1,7 +1,7 @@
 /*
  * cb.c
  *
- * Version:     $Id: cb.c,v 1.1.2.3 2006/05/01 16:48:11 aland Exp $
+ * Version:     $Id: cb.c,v 1.1.2.4 2007/04/08 14:39:26 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -48,10 +48,16 @@ void cbtls_info(const SSL *s, int where, int ret)
 			SSL_alert_type_string_long(ret),
 			SSL_alert_desc_string_long(ret));
 	} else if (where & SSL_CB_EXIT) {
-		if (ret == 0)
+		if (ret == 0) {
 			radlog(L_ERR, "%s:failed in %s\n", str, state);
-		else if (ret < 0)
-			radlog(L_ERR, "%s:error in %s\n", str, state);
+		} else if (ret < 0) {
+			if (SSL_want_read(s)) {
+				DEBUG2("%s: Need to read more data: %s",
+				       str, state);
+			} else {
+				radlog(L_ERR, "%s:error in %s\n", str, state);
+			}
+		}
 	}
 }
 
