@@ -6,7 +6,7 @@
  *
  * code common to EAP-SIM clients and to servers.
  *
- * Version:     $Id: eapsimlib.c,v 1.7.2.2 2005/01/26 20:31:31 aland Exp $
+ * Version:     $Id: eapsimlib.c,v 1.7.2.2.2.1 2007/02/15 12:51:37 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@
 #include "eap_sim.h"
 #include "sha1.h"
 
-static const char rcsid[] = "$Id: eapsimlib.c,v 1.7.2.2 2005/01/26 20:31:31 aland Exp $";
+static const char rcsid[] = "$Id: eapsimlib.c,v 1.7.2.2.2.1 2007/02/15 12:51:37 aland Exp $";
 
 /*
  * given a radius request with many attribues in the EAP-SIM range, build
@@ -343,6 +343,7 @@ int unmap_eapsim_basictypes(RADIUS_PACKET *r,
 	}
 
 	newvp = paircreate(ATTRIBUTE_EAP_SIM_SUBTYPE, PW_TYPE_INTEGER);
+	if (!newvp) return 0;
 	newvp->lvalue = attr[0];
 	newvp->length = 1;
 	pairadd(&(r->vps), newvp);
@@ -529,6 +530,7 @@ const char *subtypes[]={ "subtype0", "subtype1", "subtype2", "subtype3",
 			 "challenge",
 			 "notification",
 			 "reauth",
+			 "client-error",
 			 NULL };
 
 const char *sim_subtype2name(enum eapsim_subtype subtype,
