@@ -7,7 +7,7 @@
  * This code was written from scratch by Michael Richardson, and it is
  * dual licensed under both GPL and BSD.
  *
- * Version:     $Id: fips186prf.c,v 1.3.2.1 2005/08/24 14:37:52 nbk Exp $
+ * Version:     $Id: fips186prf.c,v 1.3.2.1.2.1 2006/05/19 14:19:15 nbk Exp $
  *
  * GPL notice:
  *
@@ -61,8 +61,6 @@
 #include <inttypes.h>
 #endif
 
-#include <rad_assert.h>
-
 #include "sha1.h"
 
 /*
@@ -266,6 +264,9 @@ main(int argc, char *argv[])
 
 /*
  * $Log: fips186prf.c,v $
+ * Revision 1.3.2.1.2.1  2006/05/19 14:19:15  nbk
+ * 	Don't use rad_assert in libeap, it's a server-only function.
+ *
  * Revision 1.3.2.1  2005/08/24 14:37:52  nbk
  * 	Fix compilation warnings with gcc 4.0
  *
