@@ -1,7 +1,7 @@
 /*
  * radeapclient.c	EAP specific radius packet debug tool.
  *
- * Version:	$Id: radeapclient.c,v 1.7.4.5 2006/05/19 14:22:23 nbk Exp $
+ * Version:	$Id: radeapclient.c,v 1.7.4.6 2007/03/05 14:09:10 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2000  Miquel van Smoorenburg <miquels@cistron.nl>
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
-static const char rcsid[] = "$Id: radeapclient.c,v 1.7.4.5 2006/05/19 14:22:23 nbk Exp $";
+static const char rcsid[] = "$Id: radeapclient.c,v 1.7.4.6 2007/03/05 14:09:10 aland Exp $";
 
 #include "autoconf.h"
 
@@ -438,7 +438,7 @@ static int process_eap_challenge(RADIUS_PACKET *req,
 	/* look for the AT_MAC and the challenge data */
 	mac   = pairfind(req->vps, ATTRIBUTE_EAP_SIM_BASE+PW_EAP_SIM_MAC);
 	randvp= pairfind(req->vps, ATTRIBUTE_EAP_SIM_BASE+PW_EAP_SIM_RAND);
-	if(mac == NULL || rand == NULL) {
+	if(mac == NULL || randvp == NULL) {
 		fprintf(stderr, "radeapclient: challenge message needs to contain RAND and MAC\n");
 		return 0;
 	}
@@ -975,7 +975,7 @@ int main(int argc, char **argv)
 			timeout = atof(optarg);
 			break;
 		case 'v':
-			printf("radclient: $Id: radeapclient.c,v 1.7.4.5 2006/05/19 14:22:23 nbk Exp $ built on " __DATE__ " at " __TIME__ "\n");
+			printf("radclient: $Id: radeapclient.c,v 1.7.4.6 2007/03/05 14:09:10 aland Exp $ built on " __DATE__ " at " __TIME__ "\n");
 			exit(0);
 			break;
                case 'S':
