@@ -1,7 +1,7 @@
 /*
  * rlm_eap_md5.c    Handles that are called from eap
  *
- * Version:     $Id: rlm_eap_md5.c,v 1.6 2004/02/26 19:04:30 aland Exp $
+ * Version:     $Id: rlm_eap_md5.c,v 1.6.4.1 2007/03/05 14:24:21 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -133,6 +133,7 @@ static int md5_authenticate(void *arg, EAP_HANDLER *handler)
 	 */
 	reply = eapmd5_alloc();
 	if (!reply) {
+		eapmd5_free(&packet);
 		return 0;
 	}
 	reply->id = handler->eap_ds->request->id;
@@ -145,6 +146,7 @@ static int md5_authenticate(void *arg, EAP_HANDLER *handler)
 	if (eapmd5_verify(packet, password, handler->opaque)) {
 		reply->code = PW_MD5_SUCCESS;
 	} else {
+		DEBUG2("  rlm_eap_md5: Passwords do not match");
 		reply->code = PW_MD5_FAILURE;
 	}
 
