@@ -1,7 +1,7 @@
 /*
  * rlm_eap_tls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: rlm_eap_tls.c,v 1.21.4.12 2006/11/29 22:59:48 aland Exp $
+ * Version:     $Id: rlm_eap_tls.c,v 1.21.4.14 2007/04/20 11:58:46 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -88,6 +88,12 @@ static int load_dh_params(SSL_CTX *ctx, char *file)
 
 	dh = PEM_read_bio_DHparams(bio, NULL, NULL, NULL);
 	BIO_free(bio);
+	if (!dh) {
+		radlog(L_INFO, "WARNING: rlm_eap_tls: Unable to set DH parameters.  DH cipher suites may not work!");
+		DEBUG2("WARNING: Fix this by running the OpenSSL command listed in eap.conf");
+		return 0;
+	}
+
 	if (SSL_CTX_set_tmp_dh(ctx, dh) < 0) {
 		radlog(L_ERR, "rlm_eap_tls: Unable to set DH parameters");
 		DH_free(dh);
@@ -501,6 +507,7 @@ static int eaptls_attach(CONF_SECTION *cs, void **instance)
 	 */
 	conf = (EAP_TLS_CONF *)malloc(sizeof(*conf));
 	if (conf == NULL) {
+		free(inst);
 		radlog(L_ERR, "rlm_eap_tls: out of memory");
 		return -1;
 	}
