@@ -1,7 +1,7 @@
 /*
  * rlm_eap_ttls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: rlm_eap_ttls.c,v 1.5.4.1 2006/10/18 21:49:47 aland Exp $
+ * Version:     $Id: rlm_eap_ttls.c,v 1.5.4.3 2007/07/13 09:37:52 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -142,12 +142,13 @@ static void ttls_free(void *p)
 
 	pairfree(&t->username);
 	pairfree(&t->state);
+	pairfree(&t->reply);
 	free(t);
 }
 
 
 /*
- *	Free the TTLS per-session data
+ *	Allocate the TTLS per-session data
  */
 static ttls_tunnel_t *ttls_alloc(rlm_eap_ttls_t *inst)
 {
@@ -191,12 +192,17 @@ static int eapttls_authenticate(void *arg, EAP_HANDLER *handler)
 		 */
 	case EAPTLS_SUCCESS:
 		if (t->authenticated) {
+			if (t->reply) {
+				pairmove(&handler->request->reply->vps,
+					 &t->reply);
+				pairfree(&t->reply);
+			}
 			eaptls_success(handler->eap_ds, 0);
 			eaptls_gen_mppe_keys(&handler->request->reply->vps,
 					     tls_session->ssl,
 					     "ttls keying material");
 		} else {
-		eaptls_request(handler->eap_ds, tls_session);
+			eaptls_request(handler->eap_ds, tls_session);
 		}
 		return 1;
 
