@@ -1,7 +1,7 @@
 /*
  * rlm_mschap.c
  *
- * Version:	$Id: rlm_mschap.c,v 1.59.2.3.2.2 2006/04/13 16:28:30 aland Exp $
+ * Version:	$Id: rlm_mschap.c,v 1.59.2.3.2.5 2007/07/04 13:25:21 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@
 
 #include	"smbdes.h"
 
-static const char rcsid[] = "$Id: rlm_mschap.c,v 1.59.2.3.2.2 2006/04/13 16:28:30 aland Exp $";
+static const char rcsid[] = "$Id: rlm_mschap.c,v 1.59.2.3.2.5 2007/07/04 13:25:21 aland Exp $";
 
 static const char *letters = "0123456789ABCDEF";
 
@@ -305,7 +305,7 @@ static int mschap_xlat(void *instance, REQUEST *request,
 {
 	int		i, data_len;
 	uint8_t		*data = NULL;
-	uint8_t		buffer[8];
+	uint8_t		buffer[32];
 	VALUE_PAIR	*user_name;
 	VALUE_PAIR	*chap_challenge, *response;
 	rlm_mschap_t	*inst = instance;
@@ -551,6 +551,39 @@ static int mschap_xlat(void *instance, REQUEST *request,
 
 		return strlen(out);
 
+		/*
+		 * Return the NT-Hash of the passed string
+		 */
+	} else if (strncasecmp(fmt, "NT-Hash ", 8) == 0) {
+		char *p;
+
+		p = fmt + 8;	/* 7 is the length of 'NT-Hash' */
+		if ((p == '\0')	 || (outlen <= 32))
+			return 0;
+		DEBUG("rlm_mschap: NT-Hash: %s",p);
+		ntpwdhash(buffer,p);
+
+		lrad_bin2hex(buffer, out, 16);
+		out[32] = '\0';
+		DEBUG("rlm_mschap: NT-Hash: Result: %s",out);
+		return 32;
+
+		/*
+		 * Return the LM-Hash of the passed string
+		 */
+	} else if (strncasecmp(fmt, "LM-Hash ", 8) == 0) {
+		char *p;
+
+		p = fmt + 8;	/* 7 is the length of 'LM-Hash' */
+		if ((p == '\0') || (outlen <= 32))
+			return 0;
+			
+		DEBUG("rlm_mschap: LM-Hash: %s",p);
+		smbdes_lmpwdhash(p,buffer);
+		lrad_bin2hex(buffer, out, 16);
+		out[32] = '\0';
+		DEBUG("rlm_mschap: LM-Hash: Result: %s",out);
+		return 32;
 	} else {
 		DEBUG2("  rlm_mschap: Unknown expansion string \"%s\"",
 		       fmt);
@@ -616,7 +649,6 @@ static int mschap_detach(void *instance){
 	free(inst->ntlm_auth);
 	if (inst->xlat_name) {
 		xlat_unregister(inst->xlat_name, mschap_xlat);
-		free(inst->xlat_name);
 	}
 	free(instance);
 	return 0;
@@ -629,7 +661,6 @@ static int mschap_detach(void *instance){
  */
 static int mschap_instantiate(CONF_SECTION *conf, void **instance)
 {
-	const char *xlat_name;
 	rlm_mschap_t *inst;
 
 	inst = *instance = rad_malloc(sizeof(*inst));
@@ -658,19 +689,17 @@ static int mschap_instantiate(CONF_SECTION *conf, void **instance)
 	/*
 	 *	Create the dynamic translation.
 	 */
-	xlat_name = cf_section_name2(conf);
-	if (xlat_name == NULL)
-		xlat_name = cf_section_name1(conf);
-	if (xlat_name){
-		inst->xlat_name = strdup(xlat_name);
-		xlat_register(xlat_name, mschap_xlat, inst);
-	}
+	inst->xlat_name = cf_section_name2(conf);
+	if (!inst->xlat_name) inst->xlat_name = cf_section_name1(conf);
+	xlat_register(inst->xlat_name, mschap_xlat, inst);
 
 	/*
 	 *	For backwards compatibility
 	 */
 	if (!dict_valbyname(PW_AUTH_TYPE, inst->xlat_name)) {
 		inst->auth_type = "MS-CHAP";
+	} else {
+		inst->auth_type = inst->xlat_name;
 	}
 
 	return 0;
@@ -796,7 +825,22 @@ static int do_mschap(rlm_mschap_t *inst,
 					     buffer, sizeof(buffer),
 					     NULL, NULL);
 		if (result != 0) {
+			char *p;
+			
 			DEBUG2("  rlm_mschap: External script failed.");
+
+			vp = pairmake("Module-Failure-Message", "", T_OP_EQ);
+			if (!vp) {
+				radlog(L_ERR, "No memory");
+				return -1;
+			}
+
+			p = strchr(buffer, '\n');
+			if (p) *p = '\0';
+			snprintf(vp->strvalue, sizeof(vp->strvalue), 
+				 "rlm_mschap: %s", buffer);
+			vp->length = strlen(vp->strvalue);
+			pairadd(&request->packet->vps, vp);
 			return -1;
 		}
 
