@@ -1,7 +1,7 @@
 /*
  * rlm_perl.c
  *
- * Version:    $Id: rlm_perl.c,v 1.13.4.7 2006/04/27 17:35:44 aland Exp $
+ * Version:    $Id: rlm_perl.c,v 1.13.4.8 2007/01/26 09:38:38 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@
 extern char **environ;
 #endif
 
-static const char rcsid[] = "$Id: rlm_perl.c,v 1.13.4.7 2006/04/27 17:35:44 aland Exp $";
+static const char rcsid[] = "$Id: rlm_perl.c,v 1.13.4.8 2007/01/26 09:38:38 aland Exp $";
 
 #ifdef USE_ITHREADS
 
@@ -693,8 +693,8 @@ static int perl_xlat(void *instance, REQUEST *request, char *fmt, char * out,
 		POPs ;
 	} else if (count > 0) {
 		tmp = POPp;
-		ret = strlen(tmp);
-		strncpy(out,tmp,ret);
+		strNcpy(out,tmp,freespace);
+		ret = strlen(out);
 
 		radlog(L_DBG,"rlm_perl: Len is %d , out is %s freespace is %d",
 		       ret, out,freespace);
