@@ -1,7 +1,7 @@
 /*
  * sql_postgresql.c		Postgresql rlm_sql driver
  *
- * Version:	$Id: sql_postgresql.c,v 1.38.4.1 2005/12/14 18:32:03 nbk Exp $
+ * Version:	$Id: sql_postgresql.c,v 1.38.4.2 2007/03/21 13:02:59 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -61,6 +61,7 @@ typedef struct rlm_sql_postgres_sock {
 /* Prototypes */
 static int sql_store_result(SQLSOCK * sqlsocket, SQL_CONFIG *config);
 static int sql_num_fields(SQLSOCK * sqlsocket, SQL_CONFIG *config);
+static int sql_close(SQLSOCK * sqlsocket, SQL_CONFIG *config);
 
 /* Internal function. Return true if the postgresql status value
  * indicates successful completion of the query. Return false otherwise
@@ -181,7 +182,7 @@ static int sql_init_socket(SQLSOCK *sqlsocket, SQL_CONFIG *config) {
 	if (PQstatus(pg_sock->conn) == CONNECTION_BAD) {
 		radlog(L_ERR, "rlm_sql_postgresql: Couldn't connect socket to PostgreSQL server %s@%s:%s", config->sql_login, config->sql_server, config->sql_db);
 		radlog(L_ERR, "rlm_sql_postgresql: Postgresql error '%s'", PQerrorMessage(pg_sock->conn));
-		PQfinish(pg_sock->conn);
+		sql_close(sqlsocket, config);
 		return SQL_DOWN;
 	}
 
@@ -414,6 +415,7 @@ static int sql_close(SQLSOCK * sqlsocket, SQL_CONFIG *config) {
 
 	if (!pg_sock->conn) return 0;
 
+	/* PQfinish also frees the memory used by the PGconn structure */
 	PQfinish(pg_sock->conn);
 	pg_sock->conn = NULL;
 
