@@ -2,7 +2,7 @@
  * rlm_sql.c		SQL Module
  * 		Main SQL module file. Most ICRADIUS code is located in sql.c
  *
- * Version:	$Id: rlm_sql.c,v 1.131.2.8.2.2 2006/05/20 12:44:37 nbk Exp $
+ * Version:	$Id: rlm_sql.c,v 1.131.2.8.2.5 2007/07/17 12:59:08 pnixon Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-	"$Id: rlm_sql.c,v 1.131.2.8.2.2 2006/05/20 12:44:37 nbk Exp $";
+	"$Id: rlm_sql.c,v 1.131.2.8.2.5 2007/07/17 12:59:08 pnixon Exp $";
 
 #include "autoconf.h"
 
@@ -138,7 +138,6 @@ static int rlm_sql_init(void) {
 /*
  *	Yucky prototype.
  */
-static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, const char *username);
 static int generate_sql_clients(SQL_INST *inst);
 static int sql_escape_func(char *out, int outlen, const char *in);
 
@@ -440,7 +439,7 @@ static int sql_escape_func(char *out, int outlen, const char *in)
  *	escape it twice. (it will make things wrong if we have an
  *	escape candidate character in the username)
  */
-static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, const char *username)
+int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, const char *username)
 {
 	VALUE_PAIR *vp=NULL;
 	char tmpuser[MAX_STRING_LEN];
@@ -566,20 +565,20 @@ static int rlm_sql_detach(void *instance)
 {
 	SQL_INST *inst = instance;
 
-	if (inst->sqlpool) {
-		sql_poolfree(inst);
-	}
-
-	if (inst->config->xlat_name) {
-		xlat_unregister(inst->config->xlat_name,sql_xlat);
-		free(inst->config->xlat_name);
-	}
-
 	paircompare_unregister(PW_SQL_GROUP, sql_groupcmp);
 
 	if (inst->config) {
 		int i;
 
+		if (inst->sqlpool) {
+			sql_poolfree(inst);
+		}
+
+		if (inst->config->xlat_name) {
+			xlat_unregister(inst->config->xlat_name,sql_xlat);
+			free(inst->config->xlat_name);
+		}
+
 		/*
 		 *	Free up dynamically allocated string pointers.
 		 */
