@@ -1,7 +1,7 @@
 /*
  *  rlm_sqlippool.c     rlm_sqlippool - FreeRADIUS SQL IP Pool Module
  *
- * Version:     $Id: rlm_sqlippool.c,v 1.3.2.3 2006/08/16 17:06:49 pnixon Exp $
+ * Version:     $Id: rlm_sqlippool.c,v 1.3.2.6 2007/07/17 18:46:32 pnixon Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
 
 #include <rlm_sql.h>
 
-static const char rcsid[] = "$Id: rlm_sqlippool.c,v 1.3.2.3 2006/08/16 17:06:49 pnixon Exp $";
+static const char rcsid[] = "$Id: rlm_sqlippool.c,v 1.3.2.6 2007/07/17 18:46:32 pnixon Exp $";
 
 
 
@@ -285,6 +285,12 @@ static int sqlippool_command(const char * fmt, SQLSOCK * sqlsocket, void * insta
 	 * Do an xlat on the provided string
 	 */
 	if (request) {
+		char sqlusername[MAX_STRING_LEN];
+
+		if(sql_set_user(data->sql_inst, request, sqlusername, NULL) < 0) {
+			return RLM_MODULE_FAIL;
+		}
+
 		if (!radius_xlat(query, sizeof(query), expansion, request, NULL)) {
 			radlog(L_ERR, "sqlippool_command: xlat failed.");
 			return 0;
@@ -323,6 +329,12 @@ static int sqlippool_query1(char * out, int outlen, const char * fmt, SQLSOCK *
 	 * Do an xlat on the provided string
 	 */
 	if (request) {
+		char sqlusername[MAX_STRING_LEN];
+
+		if(sql_set_user(data->sql_inst, request, sqlusername, NULL) < 0) {
+			return RLM_MODULE_FAIL;
+		}
+
 		if (!radius_xlat(query, sizeof(query), expansion, request, NULL)) {
 			radlog(L_ERR, "sqlippool_command: xlat failed.");
 			out[0] = '\0';
@@ -578,7 +590,7 @@ static int sqlippool_postauth(void *instance, REQUEST * request)
 
 		DEBUG("rlm_sqlippool: IP number could not be allocated.");
 		sql_release_socket(data->sql_inst, sqlsocket);
-		return RLM_MODULE_NOOP;
+		return RLM_MODULE_NOTFOUND;
 	}
 
 	ip_allocation = ip_addr(allocation);
